diff --git a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/__init__.py b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/__init__.py
index 13742a619..1ef69866c 100644
--- a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/__init__.py
+++ b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/__init__.py
@@ -1671,6 +1671,9 @@ class TableView(BaseTableView, DataTypeReader, VacuumSettings,
                 self.manager.version >= 120000:
             sub_modules.append('compound_trigger')
 
+        if self.manager.version >= 90500:
+            sub_modules.append('row_security_policy')
+
         if tid:
             status, data = self._fetch_properties(did, scid, tid)
 
diff --git a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/__init__.py b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/__init__.py
index 65dc046d3..7b666b861 100644
--- a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/__init__.py
+++ b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/__init__.py
@@ -13,7 +13,7 @@ import simplejson as json
 from functools import wraps
 
 import pgadmin.browser.server_groups.servers.databases as databases
-from flask import render_template, request, jsonify
+from flask import render_template, request, jsonify, current_app
 from flask_babelex import gettext
 from pgadmin.browser.collection import CollectionNodeModule
 from pgadmin.browser.utils import PGChildNodeView
@@ -24,6 +24,8 @@ from config import PG_DEFAULT_DRIVER
 from pgadmin.browser.server_groups.servers.databases.schemas.tables. \
     row_security_policies import utils as row_security_policies_utils
 from pgadmin.utils.compile_template_name import compile_template_path
+from pgadmin.tools.schema_diff.node_registry import SchemaDiffRegistry
+from pgadmin.tools.schema_diff.directory_compare import directory_diff
 
 
 class RowSecurityModule(CollectionNodeModule):
@@ -124,26 +126,24 @@ class RowSecurityView(PGChildNodeView):
       - This function will used to create all the child node within that
       collection. Here it will create all the policy nodes.
 
-    * properties(gid, sid, did, rg_id)
+    * properties(gid, sid, did, plid)
       - This function will show the properties of the selected policy node
 
-    * create(gid, sid, did, rg_id)
+    * create(gid, sid, did, plid)
       - This function will create the new policy object
 
-    * update(gid, sid, did, rg_id)
+    * update(gid, sid, did, plid)
       - This function will update the data for the selected policy node
 
-    * delete(self, gid, sid, rg_id):
+    * delete(self, gid, sid, plid):
       - This function will drop the policy object
 
-    * msql(gid, sid, did, rg_id)
+    * msql(gid, sid, did, plid)
       - This function is used to return modified sql for the selected
       policy node
 
-    * get_sql(data, rg_id)
-      - This function will generate sql from model data
 
-    * sql(gid, sid, did, rg_id):
+    * sql(gid, sid, did, plid):
       - This function will generate sql to show in sql pane for the selected
       policy node.
     """
@@ -226,7 +226,8 @@ class RowSecurityView(PGChildNodeView):
 
         # fetch schema name by schema id
         sql = render_template("/".join(
-            [self.template_path, 'properties.sql']), tid=tid)
+            [self.template_path, 'properties.sql']), schema=self.schema,
+            tid=tid)
         status, res = self.conn.execute_dict(sql)
 
         if not status:
@@ -297,7 +298,7 @@ class RowSecurityView(PGChildNodeView):
         properties tab
 
         """
-        status, data = self._fetch_properties(plid)
+        status, data = self._fetch_properties(did, tid, plid, scid)
         if not status:
             return data
 
@@ -306,7 +307,7 @@ class RowSecurityView(PGChildNodeView):
             status=200
         )
 
-    def _fetch_properties(self, plid):
+    def _fetch_properties(self, did, tid, plid, scid):
         """
         This function is used to fetch the properties of the specified object
         :param plid:
@@ -314,7 +315,7 @@ class RowSecurityView(PGChildNodeView):
         """
         sql = render_template("/".join(
             [self.template_path, 'properties.sql']
-        ), plid=plid, datlastsysoid=self.datlastsysoid)
+        ), plid=plid, scid=scid, datlastsysoid=self.datlastsysoid)
         status, res = self.conn.execute_dict(sql)
 
         if not status:
@@ -406,7 +407,7 @@ class RowSecurityView(PGChildNodeView):
         try:
             sql, name = row_security_policies_utils.get_sql(self.conn, data,
                                                             did,
-                                                            tid, plid,
+                                                            tid, plid, scid,
                                                             self.datlastsysoid,
                                                             self.schema,
                                                             self.table)
@@ -430,7 +431,7 @@ class RowSecurityView(PGChildNodeView):
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, tid, plid=None):
+    def delete(self, gid, sid, did, scid, tid, plid=None, only_sql=False):
         """
         This function will drop the policy object
         :param plid: policy id
@@ -487,6 +488,8 @@ class RowSecurityView(PGChildNodeView):
                                       cascade=cascade,
                                       result=result
                                       )
+                if only_sql:
+                    return sql
                 status, res = self.conn.execute_scalar(sql)
                 if not status:
                     return internal_server_error(errormsg=res)
@@ -507,7 +510,7 @@ class RowSecurityView(PGChildNodeView):
         data = dict(request.args)
 
         sql, name = row_security_policies_utils.get_sql(self.conn, data, did,
-                                                        tid, plid,
+                                                        tid, plid, scid,
                                                         self.datlastsysoid,
                                                         self.schema,
                                                         self.table)
@@ -528,7 +531,7 @@ class RowSecurityView(PGChildNodeView):
         This function will generate sql to render into the sql panel
         """
         sql = render_template("/".join(
-            [self.template_path, 'properties.sql']), plid=plid)
+            [self.template_path, 'properties.sql']), scid=scid, plid=plid)
         status, res = self.conn.execute_dict(sql)
         if not status:
             return internal_server_error(errormsg=res)
@@ -585,5 +588,123 @@ class RowSecurityView(PGChildNodeView):
             status=200
         )
 
+    @check_precondition
+    def get_sql_from_diff(self, gid, sid, did, scid, tid, plid, data=None,
+                          diff_schema=None, drop_req=False):
+
+        sql = ''
+        if data:
+            data['schema'] = self.schema
+            data['table'] = self.table
+            sql, name = row_security_policies_utils.get_sql(
+                self.conn, data, did, tid, plid, scid, self.datlastsysoid,
+                self.schema, self.table)
+
+            sql = sql.strip('\n').strip(' ')
+
+        elif diff_schema:
+            schema = diff_schema
+
+            sql = row_security_policies_utils.get_reverse_engineered_sql(
+                self.conn, schema,
+                self.table, did, tid, plid, scid,
+                self.datlastsysoid,
+                template_path=None, with_header=False)
+
+        drop_sql = ''
+        if drop_req:
+            drop_sql = '\n' + self.delete(gid=1, sid=sid, did=did,
+                                          scid=scid, tid=tid,
+                                          plid=plid)
+        if drop_sql != '':
+            sql = drop_sql + '\n\n' + sql
+        return sql
+
+    @check_precondition
+    def fetch_objects_to_compare(self, sid, did, scid, tid, oid=None):
+        """
+        This function will fetch the list of all the policies for
+        specified schema id.
+
+        :param sid: Server Id
+        :param did: Database Id
+        :param scid: Schema Id
+        :param oid: Policy Id
+        :return:
+        """
+
+        res = dict()
+
+        if not oid:
+            SQL = render_template("/".join([self.template_path,
+                                            'nodes.sql']), tid=tid)
+            status, policies = self.conn.execute_2darray(SQL)
+            if not status:
+                current_app.logger.error(policies)
+                return False
+            for row in policies['rows']:
+                status, data = self._fetch_properties(did, tid,
+                                                      row['oid'], scid)
+                if status:
+                    res[row['name']] = data
+        else:
+            status, data = self._fetch_properties(did, tid,
+                                                  oid, scid)
+            if not status:
+                current_app.logger.error(data)
+                return False
+            res = data
+
+        return res
+
+    def ddl_compare(self, **kwargs):
+        """
+        This function returns the DDL/DML statements based on the
+        comparison status.
+
+        :param kwargs:
+        :return:
+        """
+
+        src_params = kwargs.get('source_params')
+        tgt_params = kwargs.get('target_params')
+        source = kwargs.get('source')
+        target = kwargs.get('target')
+        target_schema = kwargs.get('target_schema')
+        comp_status = kwargs.get('comp_status')
+
+        diff = ''
+        if comp_status == 'source_only':
+            diff = self.get_sql_from_diff(gid=src_params['gid'],
+                                          sid=src_params['sid'],
+                                          did=src_params['did'],
+                                          scid=src_params['scid'],
+                                          tid=src_params['tid'],
+                                          plid=source['oid'],
+                                          diff_schema=target_schema)
+        elif comp_status == 'target_only':
+            diff = self.delete(gid=1,
+                               sid=tgt_params['sid'],
+                               did=tgt_params['did'],
+                               scid=tgt_params['scid'],
+                               tid=tgt_params['tid'],
+                               plid=target['oid'],
+                               only_sql=True)
+        elif comp_status == 'different':
+            diff_dict = directory_diff(
+                source, target, difference={}
+            )
+            # parse_acl(source, target, diff_dict)
+
+            diff = self.get_sql_from_diff(gid=tgt_params['gid'],
+                                          sid=tgt_params['sid'],
+                                          did=tgt_params['did'],
+                                          scid=tgt_params['scid'],
+                                          tid=tgt_params['tid'],
+                                          plid=target['oid'],
+                                          data=diff_dict)
+        return diff
+
 
+SchemaDiffRegistry(blueprint.node_type, RowSecurityView, 'table')
 RowSecurityView.register_node_view(blueprint)
diff --git a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/tests/default/test.json b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/tests/default/test.json
index f57adc262..86a60699e 100644
--- a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/tests/default/test.json
+++ b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/tests/default/test.json
@@ -36,7 +36,8 @@
       "data": {
         "name": "select_policy",
         "event": "SELECT",
-        "policyowner": "public"
+        "policyowner": "public",
+        "schema": "public"
       },
       "expected_sql_file": "create_select_policy.sql"
     },
@@ -48,7 +49,8 @@
       "data": {
         "name": "insert_policy",
         "event": "INSERT",
-        "policyowner": "public"
+        "policyowner": "public",
+        "schema": "public"
       },
       "expected_sql_file": "create_insert_policy.sql"
     },
@@ -58,7 +60,8 @@
       "endpoint": "NODE-row_security_policy.obj",
       "sql_endpoint": "NODE-row_security_policy.sql_id",
       "data": {
-        "name": "test"
+        "name": "test",
+        "schema": "public"
       },
       "expected_sql_file": "create_public_policy.sql"
     },
diff --git a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/utils.py b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/utils.py
index f61eadca1..144bc4f79 100644
--- a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/utils.py
+++ b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/row_security_policies/utils.py
@@ -61,15 +61,16 @@ def get_parent(conn, tid, template_path=None):
 
 
 @get_template_path
-def get_sql(conn, data, did, tid, plid, datlastsysoid, schema, table,
-            mode=None, template_path=None):
+def get_sql(conn, data, did, tid, plid, scid, datlastsysoid, schema, table,
+            template_path=None):
     """
     This function will generate sql from model data
     """
 
     if plid is not None:
         sql = render_template("/".join(
-            [template_path, 'properties.sql']), plid=plid)
+            [template_path, 'properties.sql']), schema=schema, plid=plid,
+            scid=scid)
         status, res = conn.execute_dict(sql)
         if not status:
             return internal_server_error(errormsg=res)
@@ -95,7 +96,7 @@ def get_sql(conn, data, did, tid, plid, datlastsysoid, schema, table,
 
 
 @get_template_path
-def get_reverse_engineered_sql(conn, schema, table, did, tid, plid,
+def get_reverse_engineered_sql(conn, schema, table, did, tid, plid, scid,
                                datlastsysoid,
                                template_path=None, with_header=True):
     """
@@ -114,24 +115,25 @@ def get_reverse_engineered_sql(conn, schema, table, did, tid, plid,
     :return:
     """
     SQL = render_template("/".join(
-        [template_path, 'properties.sql']), plid=plid)
+        [template_path, 'properties.sql']), plid=plid, scid=scid)
 
     status, res = conn.execute_dict(SQL)
     if not status:
         raise Exception(res)
 
     if len(res['rows']) == 0:
-        raise ObjectGone(_('Could not find the index in the table.'))
+        raise ObjectGone(_('Could not find the policy in the table.'))
 
     data = dict(res['rows'][0])
     # Adding parent into data dict, will be using it while creating sql
     data['schema'] = schema
     data['table'] = table
 
-    SQL, name = get_sql(conn, data, did, tid, None, datlastsysoid, schema,
+    SQL, name = get_sql(conn, data, did, tid, None, scid, datlastsysoid,
+                        schema,
                         table)
 
-    if with_header:
+    if not with_header:
         sql_header = u"-- POLICY: {0}\n\n-- ".format(data['name'])
 
         sql_header += render_template("/".join([template_path,
diff --git a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/schema_diff_utils.py b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/schema_diff_utils.py
index 55fe2b693..feaea85a3 100644
--- a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/schema_diff_utils.py
+++ b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/schema_diff_utils.py
@@ -254,7 +254,7 @@ class SchemaDiffTableCompare(SchemaDiffObjectCompare):
         target_params['diff_data'] = diff_dict
         diff = self.get_sql_from_table_diff(**target_params)
 
-        ignore_sub_modules = ['column', 'constraints', 'row_security_policy']
+        ignore_sub_modules = ['column', 'constraints']
         if self.manager.version < 100000:
             ignore_sub_modules.append('partition')
         if self.manager.server_type == 'pg' or self.manager.version < 120000:
diff --git a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/templates/row_security_policies/sql/9.5_plus/properties.sql b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/templates/row_security_policies/sql/9.5_plus/properties.sql
index a83ca056c..b89c9f899 100644
--- a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/templates/row_security_policies/sql/9.5_plus/properties.sql
+++ b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/templates/row_security_policies/sql/9.5_plus/properties.sql
@@ -8,12 +8,13 @@ SELECT
 FROM
     pg_policy pl
 JOIN pg_policies rw ON pl.polname=rw.policyname
+JOIN pg_namespace n ON n.nspname=rw.schemaname
 WHERE
 {% if plid %}
-      pl.oid = {{ plid }}
+      pl.oid = {{ plid }} and n.oid = {{ scid }};
 {% endif %}
 {% if tid %}
-      pl.polrelid = {{ tid }}
-{% endif %};
+      pl.polrelid = {{ tid }};
+{% endif %}
 
 
diff --git a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/utils.py b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/utils.py
index 445cafb2d..0bb5f6cf6 100644
--- a/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/utils.py
+++ b/web/pgadmin/browser/server_groups/servers/databases/schemas/tables/utils.py
@@ -535,7 +535,7 @@ class BaseTableView(PGChildNodeView, BasePartitionTable):
             for row in rset['rows']:
                 policy_sql = row_security_policies_utils. \
                     get_reverse_engineered_sql(
-                        self.conn, schema, table, did, tid, row['oid'],
+                        self.conn, schema, table, did, tid, row['oid'], scid,
                         self.datlastsysoid,
                         template_path=None, with_header=json_resp)
                 policy_sql = u"\n" + policy_sql
